// This file is part of Eigen, a lightweight C++ template library
// for linear algebra.
//
// Copyright (C) 2011 Benoit Jacob <jacob.benoit.1@gmail.com>
//
// This Source Code Form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

#ifndef EIGEN2_MACROS_H
#define EIGEN2_MACROS_H

#define ei_assert eigen_assert
#define ei_internal_assert eigen_internal_assert

#define EIGEN_ALIGN_128 EIGEN_ALIGN16

#define EIGEN_ARCH_WANTS_ALIGNMENT EIGEN_ALIGN_STATICALLY

#endif // EIGEN2_MACROS_H
